#!/bin/bash
# SPDX-License-Identifier: MIT

usage() {
    echo "
USAGE:
  $(basename "$0") [OPTIONS] [OUTPUT]

  OPTIONS:
    -i|--input <file>               -- name of the file for which
                                        the sbom is to be created
    -c|--config <yaml>              -- file in yaml format for creating sbom.
                                        default value is config.yaml
    -e|--external_sbom <spdx.json>  -- external .sbom.json to add to
                                        the sbom you are creating
    -o|--output <sbom file name>    -- file name of the sbom to be created
    -p|--package <package_list>     -- *package_list.txt created by build_rootfs
                                        e.g. baseos-x2-3.18.4-at.5.package_list.txt
    -h|--help                       -- this output

  OUTPUT:
    created sbom.json (<input>.sbom.json)
"
}

error() {
    echo "$@" >&2
    exit 1
}

required_input() {
    usage >&2
    error "-i|--input <file> option is a required parameter"
}

CONTAINER=sbom-make
SCRIPTDIR="$(realpath --relative-to=. "$(dirname "$0")")"
SCRIPT="$SCRIPTDIR//make_sbom.py"

ARGS=()

[ $# -eq 0 ] && required_input

# parse args
while [ $# -ne 0 ]; do
    case "$1" in
    "-i" | "--input")
        [ $# -lt 2 ] && error "$1 requires an argument"
        input="$2"
        shift
        ;;
    "-c" | "--config")
        [ $# -lt 2 ] && error "$1 requires an argument"
        ARGS+=(-c "$2")
        shift
        ;;
    "-e" | "--external_sbom")
        [ $# -lt 2 ] && error "$1 requires an argument"
        ARGS+=(-e "$2")
        shift
        ;;
    "-o" | "--output")
        [ $# -lt 2 ] && error "$1 requires an argument"
        ARGS+=(-o "$2")
        shift
        ;;
    "-p" | "--package")
        [ $# -lt 2 ] && error "$1 requires an argument"
        ARGS+=(-p "$2")
        shift
        ;;
    "-h" | "--help")
        usage
        exit 0
        ;;
    *)
        echo "Invalid option $1" >&2
        usage >&2
        exit 1
        ;;
    esac
    shift
done

[ -z "$input" ] && required_input

if ! docker image inspect "$CONTAINER" >/dev/null; then
    docker build -t "$CONTAINER" "$SCRIPTDIR" || error "Could not build container"
fi

docker run --rm -u "$(id -u)":"$(id -g)" -v "$PWD":/home/"$USER" -w /home/"$USER" \
    "$CONTAINER" python3 "$SCRIPT" -i "$input" "${ARGS[@]}"
